///////////////////////////////////////////////////////////////////////////
//
// NAME
//  LucasKanade.h -- image registration using the Lukas-Kanade algorithm
//
// SEE ALSO
//  LucasKanade.h       longer description
//
// Copyright ?Richard Szeliski, 2001.  See Copyright.h for more details
// (modified for CSE455 Winter 2003)
//
///////////////////////////////////////////////////////////////////////////

#include "../ImageLib/ImageLib.h"
#include "LucasKanade.h"
#include <math.h>

/******************* TO DO 2 *********************
 * LucasKanadeStep:
 *	INPUT:
 *		img1, img2: two images to align
 *		t: translation vector to be updated
 *	OUTPUT:
 *		solve dt such that
 *		sum of | img2(x+t[0]+dt[0],y+t[1]+dt[1]) - img1(x,y) |^2 over x,y is minimized;
 *		update t = t+dt.
 */
void LucasKanadeStep(CByteImage img1, CByteImage img2, float t[2])
{
    // Color images only, since need alpha
    CShape sh = img1.Shape();
    if (sh.nBands != 4)
        throw CError("LucasKanade only works on alpha-matted color images");

    // Transform the image, such that img2t(x,y) = img2(x+t[0],y+t[1]);
    CByteImage img2t(sh);       // the translated image
    CTransform3x3 T = CTransform3x3::Translation(t[0], t[1]);
    WarpGlobal(img2, img2t, T, eWarpInterpLinear);	

    static bool debug_warp = false;
    if (debug_warp)
    {
        WriteFile(img1 , "tmp_img1.tga");
        WriteFile(img2 , "tmp_img2.tga");
        WriteFile(img2t, "tmp_img2t.tga");
    }

	// Accumulate the matrix entries using gradients

    float ATA[2][2], ATb[2];
    memset(ATA, 0, sizeof(ATA));
    memset(ATb, 0, sizeof(ATb));

    for (int y = 1; y < sh.height-1; y++)       // ignore borders
    {
        for (int x = 1; x < sh.width-1; x++)
        {
            // Ignore "black" pixels created by
			// cylindrical warping & translation
            if (img1.Pixel (x, y, 3) != 255 ||
                img2t.Pixel(x, y, 3) != 255)
                continue;

            // Process all of the bands
            for (int k = 0; k < 3; k++)
            {
				// *** BEGIN TODO #1 ***
				// add your code to accumulate ATA and ATb entries
				// hint: you will have to look at img2t.Pixel(x+/-1, y+/-1, k)...

				// *** END TODO #1 ***
            }
        }
    }

    // Solve for the update and update the vector

	float dt[2] = {0,0};

	// *** BEGIN TODO #2
	// add your code to compute dt update from ATA and ATb
	// hint: this is the 2x2 system you solve in Lucas-Kanade
	

	// *** END TODO #2 ***

	t[0] += dt[0];
	t[1] += dt[1];
}

/******************* TO DO 3 *********************
 * PyramidalLucasKanade:
 *	INPUT:
 *		img1, img2: two images to align
 *		t: initial offset between two images (to be updated with iterative LK)
 *		nLevels: desired number of image pyramid levels
 *	OUTPUT:
 *		computes t (displacement of img2 from img1) by
 *		proceeding through image pyramid from coarse to fine level;
 *		updates t at each level by calling LucasKanadeStep
 */
void PyramidalLucasKanade(CByteImage& img1, CByteImage& img2, float t[2],
						  int nLevels, int nLucasKanadeSteps)
{
    // Form the two pyramids
    CBytePyramid p1(img1);
    CBytePyramid p2(img2);

	fprintf(stderr, "Starting t = (%.2f,%.2f)\n", t[0], t[1]);

    // Process in a coarse-to-fine hierarchy

	// *** BEGIN TODO ***
	// Scale down t for top level of pyramid
	// For each level l of pyramid from coarse to fine (nLevels-1 to 0):
	//		1. update t, the translation between two images at level l (p1[l], p2[l])
	//		   by calling LucasKanadeStep nLucasKanadeSteps times
	//		2. initialize t for next level (l-1) by scaling it up
    
	
	
	// *** END TODO ***

    fprintf(stderr, "Output t = (%.2f,%.2f)\n", t[0], t[1]);
}
