///////////////////////////////////////////////////////////////////////////
//
// NAME
//  LucasKanade.h -- image registration using the Lukas-Kanade algorithm
//
// SPECIFICATION
//  void LucasKanadeStep(CByteImage& img1, CByteImage& img2, float t[2]);
//
// PARAMETERS
//  img1, img2          source images
//  t                   translation vector (input / output)
//
// DESCRIPTION
//  These routines improve an alignment between two images using the
//  Lucas-Kanade Taylor series expansion of the registration error
//  in terms of the change to translation.
//
//  Img2 is resampled according to the translation vector.
//  Only pixels which both have full alphas contribute to the registration.
//
// SEE ALSO
//  LucasKanade.cpp     implementation
//
// Copyright  Richard Szeliski, 2001.  See Copyright.h for more details
// (modified for CSE455 Winter 2003)
//
///////////////////////////////////////////////////////////////////////////

void LucasKanadeStep(CByteImage img1, CByteImage img2, float t[2]);

void PyramidalLucasKanade(CByteImage& img1, CByteImage& img2, float t[2],
						  int nLevels, int nLucasKanadeSteps);
