///////////////////////////////////////////////////////////////////////////
//
// NAME
//  WarpCylindrical.h -- warp a flat (perspective) image into cylindrical
//      coordinates and/or undo radial lens distortion
//
// SPECIFICATION
//  CFloatImage WarpCylindricalField(CShape sh, float f, float k1, float k2)
// PARAMETERS
//  sh                  shape of destination image
//  f                   focal length, in pixels
//  k1, k2              radial distortion parameters
//
// DESCRIPTION
//  WarpCylindricalField produces a pixel coordinate image suitable
//  for correcting radial distortion in an image and/or mapping
//  the image into cylindrical or spherical coordinates (for image mosaics).
//
//  Use the output of WarpCylindricalField as the input to 
//  WarpLocal, along with a source and destination image, to actually
//  perform the warp.
//
// SEE ALSO
//  WarpCylindrical.cpp implementation
//  WarpImage.h         image warping code
//  Image.h             image class definition
//
// Copyright  Richard Szeliski, 2001.  See Copyright.h for more details
// (modified for CSE455 Winter 2003)
//
///////////////////////////////////////////////////////////////////////////

enum EWarpCylindricalMode
{
    eWarpCylFlat         = 0,    // flat output (radial distortion only)
    eWarpCylCylindrical  = 1,    // cylindrical image
    eWarpCylSpherical    = 2     // spherical image
};

CFloatImage WarpCylindricalField(CShape srcSh, CShape dstSh, float f,
                                 float k1, float k2);
